<?php
if (!defined('_PS_VERSION_'))
	exit;

class re_pays extends PaymentModule
{
  public function __construct(){
		$this->name = 're_pays';
		$this->tab = 'payments_gateways';
		$this->version = '1.0';
		$this->author = 'RájEshopů.cz @ Michael Novotný';
		$this->bootstrap=false;
    $this->api = "https://www.pays.cz/paymentorder";
    
		$this->currencies = true;
		$this->currencies_mode = 'checkbox';
    
    parent::__construct();

		$this->displayName = $this->l('Platební brána Pays');
		$this->description = $this->l('Implementuje platební metodu Pays');
		$this->confirmUninstall = $this->l('Opravdu chcete odinstalovat modul platební brány Pays?');
		
    $config = Configuration::getMultiple(array('RE_PAYS_MERCHANT_ID','RE_PAYS_SHOP_ID', 'RE_PAYS_HASH_PASS', 'RE_STATUS_CEKA', 'RE_STATUS_CHYBA', 'RE_STATUS_PRIJATO'));
    
    if(isset($config["RE_PAYS_MERCHANT_ID"]))
      $this->merchant_id = $config["RE_PAYS_MERCHANT_ID"];
    if(isset($config["RE_PAYS_SHOP_ID"]))
      $this->shop_id = $config["RE_PAYS_SHOP_ID"];
    if(isset($config["RE_PAYS_HASH_PASS"]))
      $this->hash_password = $config["RE_PAYS_HASH_PASS"];
    
    $this->config_vars = $config;
    
    if(!$this->merchant_id OR !$this->shop_id OR !$this->hash_password)
      $this->warning = $this->l('Zadejte autorizační údaje pro platební bránu Pays.cz');	 
		if (!count(Currency::checkPaymentCurrencies($this->id)))
			$this->warning = $this->l('Nebyla vybrána měna pro tento modul!');	  
	}
  
  public function install()
	{
		if (!parent::install() || !$this->registerHook('payment') || !$this->registerHook('paymentReturn'))
			return false;
  }
  
  public function getContent(){
    if((_PS_VERSION_>=1.5)){
			$smarty=$this->context->smarty;
      $cookie=$this->context->cookie;
		}else{
			global $smarty;
      global $cookie;
		};

    $smarty->assign("robraz_succes", false);
    if(Tools::isSubmit("ulozit_nastaveni")){
      Configuration::updateValue("RE_PAYS_MERCHANT_ID", Tools::getValue("merchant_id"));
      Configuration::updateValue("RE_PAYS_SHOP_ID", Tools::getValue("shop_id"));
      Configuration::updateValue("RE_PAYS_HASH_PASS", Tools::getValue("hash_pass"));
      Configuration::updateValue("RE_STATUS_CEKA", Tools::getValue("ceka_se"));
      Configuration::updateValue("RE_STATUS_CHYBA", Tools::getValue("chyba_platby"));
      Configuration::updateValue("RE_STATUS_PRIJATO", Tools::getValue("platba_prijata"));
      $smarty->assign("robraz_succes", true);
    }
    
    $shop_url=Tools::getShopDomain(true);
    if(substr($shop_url,-1,1)=="/"){
      $shop_url=substr($shop_url,0,-1);
    };
    $module_url = $shop_url."/modules/".$this->name."/";
    if((_PS_VERSION_>=1.6)){
			$mod_url_fail = Context::getContext()->link->getModuleLink('re_pays','fail_payment',array());
      $mod_url_success = Context::getContext()->link->getModuleLink('re_pays','success_payment',array());
      $mod_callback = Context::getContext()->link->getModuleLink('re_pays','callback',array());
		}elseif((_PS_VERSION_>=1.5)){
			$mod_url_fail = Context::getContext()->link->getModuleLink('re_pays','fail_payment',array());
      $mod_url_success = Context::getContext()->link->getModuleLink('re_pays','success_payment',array());
      $mod_callback = Context::getContext()->link->getModuleLink('re_pays','callback',array());
    }else{
			$mod_url_fail = $module_url."fail_payment.php";
      $mod_url_success = $module_url."success_payment.php";
      $mod_callback = $module_url."callback.php";
    };
    
    $smarty->assign("fail_url", $mod_url_fail);
    $smarty->assign("success_url", $mod_url_success);
    $smarty->assign("callback_url", $mod_callback);
    
    $stavy = Db::getInstance()->executeS("SELECT * FROM `"._DB_PREFIX_."order_state` a LEFT JOIN `"._DB_PREFIX_."order_state_lang` b ON(a.`id_order_state`=b.`id_order_state`) WHERE b.`id_lang`='".$cookie->id_lang."'");
    $smarty->assign("stavy", $stavy);
    $smarty->assign("form_action", Tools::htmlentitiesUTF8($_SERVER['REQUEST_URI']));
    return $this->display(__FILE__, "/views/config.tpl");
  }
  
  public function checkCurrency($cart)
	{
		
		$currency_order = new Currency($cart->id_currency);
		$currencies_module = $this->getCurrency($cart->id_currency);

		if (is_array($currencies_module)){
			foreach ($currencies_module as $currency_module)
				if ($currency_order->id == $currency_module['id_currency'])
					return true;
			}else{
				if ($currency_order->id == $currencies_module->id_currency OR $currency_order->id == $currencies_module->id)
					return true;
			};
		return false;
	}
  
  public function hookPayment($params){
		if((_PS_VERSION_>=1.5)){
			$smarty=$this->context->smarty;
		}else{
			global $smarty;
		};
		
		if((_PS_VERSION_>=1.6)){
			$smarty->assign("ps","1.6");
		}elseif((_PS_VERSION_>=1.5)){
			$smarty->assign("ps","1.5");
		}else{
			$smarty->assign("ps","1.4");
		};
		
		if (!$this->active)
			return;
		
		if (!$this->checkCurrency($params['cart']))
			return;
			
		$smarty->assign(array(
			'this_path' => $this->_path,
			'this_path_bw' => $this->_path,
			'this_path_ssl' => Tools::getShopDomainSsl(true, true).__PS_BASE_URI__.'modules/'.$this->name.'/'
		));
		return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
	}
  
  public function execPayment(){
    if((_PS_VERSION_>=1.5)){
			$cart=&$this->context->cart;
			$smarty=&$this->context->smarty;
      $cookie=&$this->context->cookie;
		}else{
			global $smarty, $cart, $cookie;
		};
	
		if((_PS_VERSION_>=1.6)){
			$smarty->assign("ps","1.6");
		}elseif((_PS_VERSION_>=1.5)){
			$smarty->assign("ps","1.5");
		}else{
			$smarty->assign("ps","1.4");
		};
	   
    if ($cart->id_customer == 0 OR $cart->id_address_delivery == 0 OR $cart->id_address_invoice == 0 OR !$this->active)
    	Tools::redirectLink(__PS_BASE_URI__.'order.php?step=1');
    
    // Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
    $authorized = false;
    foreach (Module::getPaymentModules() as $module)
    	if ($module['name'] == 're_pays')
    	{
    		$authorized = true;
    		break;
    	}
    if (!$authorized)
    	die(Tools::displayError('This payment method is not available.'));
      

    $customer = new Customer((int)$cart->id_customer);

    if (!Validate::isLoadedObject($customer))
    	Tools::redirectLink(__PS_BASE_URI__.'order.php?step=1');
    
    $currency = new Currency($cart->id_currency);
    $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
    
    //Odeslani objednavky 
    $this->validateOrder($cart->id, Configuration::get("RE_STATUS_CEKA"), $total, $this->displayName, NULL, $mailVars, (int)$currency->id, false, $customer->secure_key);
    $order = new Order($this->currentOrder);
    if (!Validate::isLoadedObject($order))
      die("Došlo k chybě při vytváření objednávky. Kontaktujte technickou podporu.");
    //Priprava presmerovani na Pays.cz
    $datas = array(
      "Merchant" => Configuration::get("RE_PAYS_MERCHANT_ID"),
      "Shop" => Configuration::get("RE_PAYS_SHOP_ID"),
      "Currency" => strtoupper($currency->iso_code),
      "Amount" => ($total*100),
      "MerchantOrderNumber" => $this->currentOrder,
      "Email" => $customer->email,
    );
    
    
    Tools::redirectAdmin($this->api."?".http_build_query($datas));
  }
  
  public function callback_fce(){
    //file_put_contents(dirname(__FILE__)."/callback.txt", "GET\n".var_export($_GET,true)."\nPOST\n".var_export($_POST,true));
    if(!empty($_GET)){
      if(!isset($_GET["hash"])){
        return "ERROR - Nebyl přijat kontrolní součet od Pays.";
      }
      $control_hash = hash_hmac("md5", $_GET["PaymentOrderID"].$_GET["MerchantOrderNumber"].$_GET["PaymentOrderStatusID"].$_GET["CurrencyID"].$_GET["Amount"].$_GET["CurrencyBaseUnits"], $this->hash_password);
      if($control_hash == $_GET["hash"]){
        if($_GET["PaymentOrderStatusID"] == "2"){
          $order = new Order($_GET["MerchantOrderNumber"]);
          $history = new OrderHistory();
  				$history->id_order = (int)($order->id);
  				if((_PS_VERSION_>=1.5)){
      			$history->changeIdOrderState(Configuration::get('RE_STATUS_CHYBA'), $order);
      		}else{
      			$history->changeIdOrderState(Configuration::get('RE_STATUS_CHYBA'), $order->id);
      		};
  				$history->addWithemail();
          //fail
          return "OK - Nastavení úspěšně provedeno. Objednávka NEBYLA zaplacena. ID objednávky: ".$order->id." a ID stavu: ".Configuration::get('RE_STATUS_CHYBA');
        }else{
          $order = new Order($_GET["MerchantOrderNumber"]);
          $history = new OrderHistory();
  				$history->id_order = (int)($order->id);
  				if((_PS_VERSION_>=1.5)){
      			$history->changeIdOrderState(Configuration::get('RE_STATUS_PRIJATO'), $order);
      		}else{
      			$history->changeIdOrderState(Configuration::get('RE_STATUS_PRIJATO'), $order->id);
      		};
  				$history->addWithemail();
          //success
          return "OK - Nastavení úspěšně provedeno. Objednávka je zaplacena. ID objednávky: ".$order->id." a ID stavu: ".Configuration::get('RE_STATUS_PRIJATO');
        }
      }else{
        return "ERROR - Nepovedlo se ověřit kontrolní součet. Rozdíl mezi zaslaným hash a výpočtem obchodu.";
      };
    }else{
      return "ERROR - Nebyla přijata žádná data.";
    };
  }
  
  public function success_payment(){
    if(!empty($_GET)){
      $order = new Order($_GET["MerchantOrderNumber"]);
      $customer = new Customer($order->id_customer);
      Tools::redirectLink(__PS_BASE_URI__.'order-confirmation.php?id_cart='.$order->id_cart.'&id_module='.$this->id.'&id_order='.$order->id.'&key='.$customer->secure_key);
    }else{
      die("Došlo k neznámé chybě");
    };
  }
  
  public function fail_payment(){
    if((_PS_VERSION_>=1.5)){
			$cart=&$this->context->cart;
			$smarty=&$this->context->smarty;
		}else{
			global $smarty, $cart;
		};
	
		if((_PS_VERSION_>=1.6)){
			$smarty->assign("ps","1.6");
		}elseif((_PS_VERSION_>=1.5)){
			$smarty->assign("ps","1.5");
		}else{
			$smarty->assign("ps","1.4");
		};
    
    return $this->display(__FILE__, 'views/templates/front/payment_fail.tpl');
  }
  
  public function hookPaymentReturn($params)
	{
		if((_PS_VERSION_>=1.5)){
			$smarty=$this->context->smarty;
		}else{
			global $smarty, $cart;
		};
		if (!$this->active)
			return;
	
		if((_PS_VERSION_>=1.5)){
			$cookie=$this->context->cookie;
		}else{
			global $cookie;
		};

		$state = $params['objOrder']->getCurrentState();
			$smarty->assign(array(
				'total_to_pay' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
				'status' => 'ok',
				'id_order' => $params['objOrder']->id
			));
			if (isset($params['objOrder']->reference) && !empty($params['objOrder']->reference))
				$this->smarty->assign('reference', $params['objOrder']->reference);
		return $this->display(__FILE__, 'views/templates/hook/payment_return.tpl');
	}
}
?>